/* Battle Pong - a pong game with some extra features */
/* Copyright (C) 2003 Dana Olson                                            */
/* For more info, read the LICENSE file                             */

#include "game.h"

SDL_Surface *sprites;
SDL_Surface *bgGame;
SND_Data hitSound, wallSound;
int resetGame(1);

void startGame()
{
    if (loadImages() != 0) {
        printf("Error loading images. Aborting.\n");
        exit(1);
    }

    if (loadSounds() != 0) {
        printf("Error loading sounds. Aborting.\n");
        exit(1);
    }

    quitFlag = 0;

    /* Reset all game data */
    resetAll();

    while (quitFlag==0) /* Main Game Loop */
    {
        updateInputs(); /* Update the input devices                                                                     */
        updateState();    /* This takes care of calling the movement routines and so on */
        checkWinConditions(); /* Duh */
        drawScreen();     /* This takes care of drawing everything onto the screen.         */
    } /* End Main Game Loop */

    /* If someone won the game, do the gameover screen */
    if (quitFlag==1) {
        /* Show the game over screen now */
        drawGameOver(ONE);
    } else if (quitFlag==2) {
        /* Show the game over screen now */
        drawGameOver(TWO);
    } else { Mix_FadeOutMusic(1000); } /* Fade music for one second */

    /* Finish playing last sound */
    while(Mix_Playing(-1)>0) { SDL_Delay(100); }

    /* Free all sound buffers */
    Mix_FreeChunk(hitSound.sample); hitSound.sample = NULL;
    Mix_FreeChunk(wallSound.sample); wallSound.sample = NULL;

    /* Free all the surfaces when we're done */
    SDL_FreeSurface(scoreText[ONE]);
    SDL_FreeSurface(scoreText[TWO]);
    SDL_FreeSurface(bgGame);
    SDL_FreeSurface(sprites);
}

void resetAll()
{
    Players[ONE].reset();
    Players[TWO].reset();
    resetGame=1; /* This fixes the TimeScaler problem I was having in updateState() */
    initPlayers(); /* This will reset the pong ball and all dimensions of player/ball sprites */
}

void checkWinConditions()
{
    if (Players[ONE].getScore()>=MAXSCORE) { quitFlag=1; }
    if (Players[TWO].getScore()>=MAXSCORE) { quitFlag=2; }
}

void updateInputs()
{
    Uint8 *keys;
    /* Someone on IRC told me to do this because of memory leaks */
    { SDL_Event ev; while(SDL_PollEvent(&ev)); }

    SDL_PumpEvents();
    keys = SDL_GetKeyState(NULL);

/* Keyboard movement */
    if (keys[KEY_P1_DOWN]) {
	Players[ONE].setSpeed(PSPEEDMAX);}
    else if (keys[KEY_P1_UP]){
	Players[ONE].setSpeed(-PSPEEDMAX);}
    else {
	Players[ONE].setSpeed(0);}

	if(numPlayers==2){
		if (keys[KEY_P2_DOWN]) {
		Players[TWO].setSpeed(PSPEEDMAX);}
		else if (keys[KEY_P2_UP]) {
		Players[TWO].setSpeed(-PSPEEDMAX);}
		else {
		Players[TWO].setSpeed(0);}
	}
	
	else if(numPlayers==1){
	aiUpdate(); /* Unless it's a single-player game, in which case, the AI handles it */
	}

    if (PongBall.isPaused())
    {
		if (getRandom(2)>2) { PongBall.xSpeed=-BSPEEDMAX; } /* Random chance for AI to win draw */
		if (getRandom(2)==2) { PongBall.ySpeed=-BSPEEDMAX; }
    }

    if (keys[KEY_EXIT]) { quitFlag=-1; }
}

void aiUpdate()
{
    int midBall = PongBall.location.y+PongBall.location.h/2;
    int midPaddle = Players[TWO].location.y+Players[TWO].location.h/2;
    int diff=abs(midPaddle-midBall);
    int movement=(PSPEEDMAX);

    /* This should stop the shakey AI */
    if (diff<PSPEEDMAX) { movement=diff; }

    if (PongBall.isPaused()) {
        /* Center the paddle if the ball is stopped */
        if ((SCREEN_HIGH/2)>midPaddle) { Players[TWO].setSpeed(movement); }
        else if ((SCREEN_HIGH/2)<midPaddle) { Players[TWO].setSpeed(-movement); }
        else { Players[TWO].setSpeed(0); }
    } else {
        /* Follow the ball as closely as possible */
        if (midBall>midPaddle) { Players[TWO].setSpeed(movement); }
        else if (midBall<midPaddle) { Players[TWO].setSpeed(-movement); }
        else { Players[TWO].setSpeed(0); }
    }
}

void updateState()
{
    /* for keeping track of timing */
    static int lastTime(0);
    int thisTime = SDL_GetTicks();

    if (resetGame!=0) {    /* safety precaution */
        lastTime = thisTime-30;
        resetGame=0; /* Fixed the bug I had with the TimeScaler */
    }

    thisTime = SDL_GetTicks();
    timeScaler = (float)(thisTime-lastTime)/60;

    Players[ONE].move(timeScaler);
    Players[TWO].move(timeScaler);
    int result=PongBall.move(timeScaler);
    if (result>0) {
        Players[result-1].incScore();
    } else if (result<0) {
        if (playSound(&wallSound)!=0) { printf("Mix_PlayChannel: %s\n",Mix_GetError()); }
    }

    if (collisionDetectRIRmW(Players[ONE].location, PongBall.location, -PongBall.location.w, 0))
    { PongBall.bounceX((int)Players[ONE].location.x+Players[ONE].location.w);
        /* Set the angle then play the sound */
        hitSound.angle=270;
        if (playSound(&hitSound)!=0) { printf("Mix_PlayChannel: %s\n",Mix_GetError()); }
    }

    if (collisionDetectRIRmW(Players[TWO].location, PongBall.location, 0, PongBall.location.w))
    { PongBall.bounceX((int)Players[TWO].location.x-PongBall.location.w);
        /* Set the angle then play the sound */
        hitSound.angle=90;
        if (playSound(&hitSound)!=0) { printf("Mix_PlayChannel: %s\n",Mix_GetError()); }
    }

    lastTime = thisTime;
}

void drawScreen()
{
    drawBackground();
    drawHUD();
    drawPlayers();
    drawBall();
    if (PongBall.isPaused()) { drawFaceoff(); }
    SDL_Flip(screen);
}

void drawFaceoff()
{
    if ((PongBall.isPaused()<=INITIALDELAY+1) && ((Players[ONE].getScore()<MAXSCORE)&&(Players[TWO].getScore()<MAXSCORE))) {
        src.x=1; src.y=127; src.w=139; src.h=27;
    } else {
        src.x=142; src.y=127; src.w=83; src.h=27;
    }
    dest=src; dest.x=SCREEN_WIDE/2-dest.w/2; dest.y=SCREEN_HIGH-dest.h-2;
    SDL_BlitSurface(sprites, &src, screen, &dest);
}

void drawBackground()
{
    src.x=0; src.y=0; src.w=bgGame->w; src.h=bgGame->h;
    dest=src;
    SDL_BlitSurface(bgGame, &src, screen, &src);
}

void drawHUD()
{
    char numVal[8];

    /* Some guy on IRC said this would work - don't ask me */
    sprintf(numVal, "%i", Players[ONE].getScore());
    /* Print the text onto the surface now */
    scoreText[ONE] = TTF_RenderText_Solid(font,numVal,txtClr);
    dest.x = 112; dest.y = -1; dest.w = scoreText[ONE]->w; dest.h = scoreText[ONE]->h;
    SDL_BlitSurface(scoreText[ONE],NULL,screen,&dest); /* Draw player one's score */

    sprintf(numVal, "%i", Players[TWO].getScore());
    /* Print the text onto the surface now */
    scoreText[TWO] = TTF_RenderText_Solid(font,numVal,txtClr);
    dest.x = SCREEN_WIDE-dest.x-scoreText[TWO]->w; dest.y = 0; dest.w = scoreText[TWO]->w; dest.h = scoreText[TWO]->h;
    SDL_BlitSurface(scoreText[TWO],NULL,screen,&dest); /* Draw player two's score */
}

void drawPlayers()
{
    SDL_BlitSurface(sprites, &Players[ONE].sprDims, screen, &Players[ONE].location);
    SDL_BlitSurface(sprites, &Players[TWO].sprDims, screen, &Players[TWO].location);
}

void drawBall()
{
    SDL_BlitSurface(sprites, &PongBall.sprDims, screen, &PongBall.location);
}

void drawGameOver(int playerNum)
{
    src.x=23; src.y=102; src.w=11; src.h=src.w; dest=src;
    for(int x(0); x < SCREEN_WIDE; x+=src.w)
    {
        for (int y(0); y < SCREEN_HIGH; y+=src.h)
        {
            dest.x=x;dest.y=y;
            SDL_BlitSurface(sprites, &src, screen, &dest);
        }
    }

    src.x=49; src.y=1; src.w=349; src.h=27; dest=src;
    dest.x=SCREEN_WIDE/2-dest.w/2; dest.y=SCREEN_HIGH/2-dest.h-5;
    SDL_BlitSurface(sprites, &src, screen, &dest);

    dest.h=src.h; src.y=58-(playerNum*(src.h+1));
    dest.y=SCREEN_HIGH/2+dest.h+5;
    SDL_BlitSurface(sprites, &src, screen, &dest);

    SDL_Flip(screen);
    SDL_Delay(5000);
}

int loadImages()
{
    /* Load the background PNG file */
    bgGame = IMG_Load("img/game.png");
    if (bgGame == NULL) {
        printf("Unable to load a necessary PNG file.\n");
        return 1;
    }

    /* Load the sprites PNG file */
    sprites = IMG_Load("img/game-sprites.png");
    if (sprites == NULL) {
        printf("Unable to load a necessary PNG file.\n");
        return 1;
    }
return 0;
}

int loadSounds()
{
    /* Load the sounds up into the buffers */
    hitSound.sample=Mix_LoadWAV("snd/bounce.wav");
    if(hitSound.sample == NULL) {
        printf("Mix_LoadWAV error occurred: %s\n", Mix_GetError());
        return 1;
    }
    hitSound.volume=255; hitSound.angle=0;
    hitSound.distance=0; hitSound.channel=1;
    hitSound.loops=0;

    wallSound.sample=Mix_LoadWAV("snd/bounce.wav");
    if(wallSound.sample == NULL) {
        printf("Mix_LoadWAV error occurred: %s\n", Mix_GetError());
        return 1;
    }

    wallSound.volume=255; wallSound.angle=0;
    wallSound.distance=0; wallSound.channel=2;
    wallSound.loops=0;

return 0;
}
